import {DiscordUser} from "../../src/wrappers/discord/DiscordUser";
import {DiscordTextChannel} from "../../src/wrappers/discord/DiscordTextChannel";
import {DiscordServer} from "../../src/wrappers/discord/DiscordServer";

const assert = require("assert");

describe("The class DiscordUser", function() {
	beforeEach(function() {
		this._member = {
			user: {
				id: 123,
				username: "name",
				guild: {},
				tag: "name#123"
			}
		};
		this.User = new DiscordUser(this._member);
	});

	afterEach(function() {
		this.User = null;
	});

	describe("The method getId", function() {
		it("Returns the user id", function() {
			//Act
			let userId: number = this.User.getId();

			//Assert
			assert.strictEqual(userId, 123, "The correct user id was returned.");
		});
	});

	describe("The method getName", function() {
		it("Returns the username", function() {
			//Act
			let username: number = this.User.getName();

			//Assert
			assert.strictEqual(username, "name", "The correct username was returned.");
		});
	});

	describe("The method getTag", function() {
		it("Returns the user tag", function() {
			//Act
			let tag: number = this.User.getTag();

			//Assert
			assert.strictEqual(tag, "name#123", "The correct tag was returned.");
		});
	});

	describe("The method getServer", function() {
		it("Returns the wrapped server object", function() {
			//Act
			let Server: any = this.User.getServer();

			//Assert
			assert.strictEqual(Server instanceof DiscordServer, true, "The server object was wrapped correctly.");
		});

		it("Returns null because the user is not acting on a server", function() {
			//Arrange
			this._member.user.guild = null;

			//Act
			let Server: any = this.User.getServer();

			//Assert
			assert.strictEqual(Server, null, "There is no server, so null was returned.");

			//Cleanup
		});
	});

	describe("The method createDM", function() {
		it("Resolves the promise and returns the channel", function() {
			//Arrange
			let _member: object = {
				user: {
					createDM: function() {
						return new Promise(resolve => {
							resolve({});
						});
					}
				}
			};
			let User: DiscordUser = new DiscordUser(_member);

			//Act
			return User.createDM().then(resolve => {
				//Assert
				assert.strictEqual(resolve instanceof DiscordTextChannel, true, "The promise returned a wrapped DiscordTextChannel.");
			});
		});

		it("Rejects the promise and returns the reason", function() {
			//Arrange
			let _member: object = {
				user: {
					createDM: function() {
						return new Promise((resolve, reject) => {
							reject("some reason");
						});
					}
				}
			};
			let User: DiscordUser = new DiscordUser(_member);

			//Act
			return User.createDM().catch(reject => {
				//Assert
				assert.strictEqual(reject, "some reason", "The promise returned the reason for its rejection.");
			});
		});
	});

	describe("The method deleteDM", function() {
		it("Resolves the promise and returns the channel", function() {
			//Arrange
			let _member: object = {
				user: {
					deleteDM: function() {
						return new Promise(resolve => {
							resolve({});
						});
					}
				}
			};
			let User: DiscordUser = new DiscordUser(_member);

			//Act
			return User.deleteDM().then(resolve => {
				//Assert
				assert.strictEqual(resolve instanceof DiscordTextChannel, true, "The promise returned a wrapped DiscordTextChannel.");
			});
		});

		it("Rejects the promise and returns the reason", function() {
			//Arrange
			let _member: object = {
				user: {
					deleteDM: function() {
						return new Promise((resolve, reject) => {
							reject("some reason");
						});
					}
				}
			};
			let User: DiscordUser = new DiscordUser(_member);

			//Act
			return User.deleteDM().catch(reject => {
				//Assert
				assert.strictEqual(reject, "some reason", "The promise returned the reason for its rejection.");
			});
		});
	});
});
